# Copyright (C) 2009 Canonical Ltd
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""Version Control for Human Beings.

Bazaar Explorer is a desktop application providing an easy-to-use
interface to the Bazaar version control system. It is designed to
be simple enough for casual users of version control, yet powerful
enough for experts to use as their primary interface to Bazaar.

Please visit the home page, http://doc.bazaar-vcs.org/explorer/en/,
for more information.
"""

version_info = (0, 8, 3, 'final', 0)
__version__ = '.'.join(map(str, version_info))


import sys

import bzrlib
import bzrlib.api
from bzrlib.commands import Command, register_command
from bzrlib.option import Option, RegistryOption

from bzrlib.lazy_import import lazy_import
lazy_import(globals(), '''
from PyQt4 import QtGui, QtCore

from bzrlib import bzrdir, debug, ui

from bzrlib.plugins.explorer.lib import (
    checkout_dialog,
    explorer,
    workspace_dialogs,
    workspace_models,
    )
''')

# These can't be lazily imported currently
from bzrlib.plugins.qbzr.lib.commands import (
    simple_revision_option,
    )


if getattr(sys, "frozen", None):
    # Add our required extra libraries for the standalone bzr.exe to path
    import bzrlib.plugins.qbzr.lib


def load_tests(basic_tests, module, loader):
    from bzrlib.plugins.explorer.tests import load_tests
    return load_tests(basic_tests, module, loader)


def _init_explorer_command(zzz=False, experimental=False):
    # Set up global exception handling.
    from bzrlib.plugins.qbzr.lib.trace import excepthook
    if 'error' not in debug.debug_flags:
        sys.excepthook = excepthook

    try:
        from bzrlib.plugins.qbzr.lib.trace import set_file_bugs_url
        set_file_bugs_url(
                    "https://bugs.launchpad.net/bzr-explorer/+filebug")
    except ImportError:
        # This function is only availible from qbzr 0.13
        pass

    from bzrlib.plugins.explorer.lib import i18n
    if not zzz:
        i18n.install()
    else:
        i18n.install_zzz()
    if experimental:
        from bzrlib.plugins.explorer import lib
        lib.set_experimental_mode(True)


class cmd_explorer(Command):
    """Desktop application for Bazaar.

    If a location is given, that location is opened. Otherwise,
    Bazaar Explorer is started with a "Welcome" page.

    See http://doc.bazaar-vcs.org/explorer/en/ for our vision,
    an overview of features, a visual tour and additional
    documentation.

    :Troubleshooting:

     A diagnostic mode is supported where commands are displayed
     in a dialog instead of being executed. This can be enabled from
     the command line by using the --dry-run option and toggled from
     within the application using View --> Diagnostic Mode.

     If started with -Derror, exceptions are displayed on the console
     instead of being trapped and converted to an error dialog. This
     can be useful when providing additional information for bug
     reports.
    """

    takes_options = [
        Option("hat", type=str,
            help="Additional set of bookmarks and tools to load.",
            ),
        Option("dry-run",
            help="Start with diagnostic mode enabled.",
            ),
        RegistryOption.from_kwargs("desktop",
            help="Desktop environment to use.",
            gnome="GNOME",
            kde="K Desktop Environment",
            windows="Windows",
            osx="Mac OS X",
            value_switches=True,
            ),
        ]
    if bzrlib.api.get_minimum_api_version(bzrlib) >= (1,14):
        takes_options.extend([
            Option("zzz", hidden=True,
                help="Special zzz translation used for debugging i18n stuff.",
                ),
            Option("experimental", hidden=True,
                help="Enable experimental features.",
                ),
            ])
    takes_args = ['location?']
    aliases = ['explore']

    def run(self, location=None, hat=None, dry_run=False,
            desktop=None, zzz=False, experimental=False):
        from bzrlib.plugins.qbzr.lib import uifactory
        _init_explorer_command(zzz, experimental)

        # Start the application
        app = QtGui.QApplication(sys.argv)
        ui.ui_factory = uifactory.QUIFactory()
        window = explorer.QExplorerMainWindow(location=location, profile=hat,
            desktop=desktop, dry_run=dry_run)
        window.show()
        app.exec_()


# Define options shared across commands below
_model_option = RegistryOption.from_kwargs("model",
    help="How to setup the repository, branch and tree.",
    feature_branches="Create a shared repository at location"
        " then create a trunk branch inside it (default).",
    shared_tree="Create a (no-trees) shared repository at location,"
        " then create a trunk branch and separate working tree"
        " (lightweight checkout) inside it.",
    plain_branch="Create a plain branch at location; if inside a"
        " shared repository, that will be used, otherwise the branch will"
        " be a standalone branch.",
    shared_repo="Create a shared repository at location.",
    shared_repo_no_trees="Create a shared repository at location"
        " and configure it so branches have no trees.",
    value_switches=True,
    )
_create_prefix_option = Option('create-prefix',
   help='Create the path leading up to the location '
        'if it does not already exist.')


class cmd_init_ala_explorer(Command):
    """Setup a repository, branch and working tree for common workspace models."""
    takes_options = [_model_option, _create_prefix_option,
        RegistryOption('format',
                help='Specify a format for the repository, etc. '
                'See "help formats".',
                lazy_registry=('bzrlib.bzrdir', 'format_registry'),
                converter=lambda name: bzrdir.format_registry.make_bzrdir(name),
                value_switches=True,
                title="Storage Format",
                ),
        ]
    takes_args = ['location?']
    aliases = ['ws-init']
    hidden=True
    def run(self, location=u'.', model=None, create_prefix=False, format=None):
        workspace_models.init_workspace(location, model=model, format=format,
            create_parent_dirs=create_prefix, outf=self.outf)


class cmd_qinit_ala_explorer(Command):
    """Setup a repository, branch and working tree using a GUI for common workspace models."""
    # TODO: add ui_mode option
    takes_options = [_model_option, _create_prefix_option]
    if bzrlib.api.get_minimum_api_version(bzrlib) >= (1,14):
        takes_options.extend([
            Option("zzz", hidden=True,
                help="Special zzz translation used for debugging i18n stuff.",
                ),
            Option("experimental", hidden=True,
                help="Enable experimental features.",
                ),
            ])
    takes_args = ['location?']
    aliases = []
    hidden=True
    def run(self, location=u'.', model=None, create_prefix=False,
        ui_mode=True, zzz=None, experimental=False):
        _init_explorer_command(zzz, experimental)

        # Start the application
        app = QtGui.QApplication(sys.argv)
        window = workspace_dialogs.QInitWorkspaceDialog(location=location,
            model=model, create_parent_dirs=create_prefix, ui_mode=ui_mode)
        window.show()
        app.exec_()


class cmd_qcheckout_ala_explorer(Command):
    """Checkout a branch. The checkout is always lightweight."""
    # TODO: add ui_mode option
    takes_options = [simple_revision_option]
    if bzrlib.api.get_minimum_api_version(bzrlib) >= (1,14):
        takes_options.extend([
            Option("zzz", hidden=True,
                help="Special zzz translation used for debugging i18n stuff.",
                ),
            ])
    takes_args = ['branch_location?', 'to_location?']
    aliases = []
    hidden=True
    def run(self, branch_location=None, to_location=None, revision=None,
        ui_mode=True, zzz=None):
        _init_explorer_command(zzz)

        # Start the application
        app = QtGui.QApplication(sys.argv)
        window = checkout_dialog.QCheckoutExplorerStyleDialog(
            branch_location=branch_location, to_location=to_location,
            revision=revision, ui_mode=ui_mode)
        window.show()
        app.exec_()


register_command(cmd_explorer)
register_command(cmd_init_ala_explorer)
register_command(cmd_qinit_ala_explorer)
register_command(cmd_qcheckout_ala_explorer)
