# Copyright (C) 2009 Canonical Ltd
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""A registry of interesting kinds and their icons."""

from cgi import escape
from PyQt4 import QtCore, QtGui

from bzrlib.plugins.explorer.lib.i18n import gettext, N_


# Kinds of locations
BRANCH_KIND = "branch"
BOUND_BRANCH_KIND = "bound-branch"
CHECKOUT_KIND = "checkout"
REPOSITORY_KIND = "repository"
ALL_KIND = "all"

# Other kinds of pages
WELCOME_PAGE = "welcome"

# Kinds of overall status
ERROR_STATUS = "error"
WARNING_STATUS = "warning"
GOOD_STATUS = "information"

# Kinds of tools
LINK_TOOL = "link"
BZR_TOOL = "bzr"
APPLICATION_TOOL = "application"

# Kinds of logos
SMALL_LOGO = "small"
MEDIUM_LOGO = "medium"
LARGE_LOGO = "large"

# Kinds of actions
ADD_ACTION = "add"
BRANCH_ACTION = BRANCH_KIND
CANCEL_ACTION = "cancel"
COMMIT_ACTION = "commit"
CONFIGURATION_ACTION = "configuration"
CONFLICTS_ACTION = "conflicts"
DELETE_ACTION = "delete"
DIFF_ACTION = "diff"
DOWNLOAD_ACTION = "download"
EDIT_ACTION = "edit"
INITIALIZE_ACTION = "initialize"
LOG_ACTION = "log"
OPEN_ACTION = "open"
OPEN_LOCATION_ACTION = "openlocation"
OPEN_FOLDER_ACTION = "open-folder"
PREFERENCES_ACTION = "preferences"
SEND_ACTION = "send"
START_ACTION = "start"

# Kinds of view
# (Note: some explicitly aliased to 'matching' commands)
GENERIC_VIEW = "generic-view"
STATUS_VIEW = COMMIT_ACTION
SUBMIT_VIEW = SEND_ACTION


_RESOURCE_BY_KIND = {
    # Locations
    BRANCH_KIND:       ":/tango/16x16/places/folder.png",
    BOUND_BRANCH_KIND: ":/tango/16x16/status/network-idle.png",
    CHECKOUT_KIND:     ":/tango/16x16/actions/media-eject.png",
    REPOSITORY_KIND:   ":/tango/16x16/mimetypes/package-x-generic.png",
    ALL_KIND:          ":/tango/16x16/mimetypes/text-x-generic.png",
    # Pages
    WELCOME_PAGE:      ":/tango/16x16/places/start-here.png",
    # Status
    ERROR_STATUS:       ":/tango/16x16/status/dialog-error.png",
    WARNING_STATUS:     ":/tango/16x16/status/dialog-warning.png",
    GOOD_STATUS:        ":/tango/16x16/status/dialog-information.png",
    # Tools
    LINK_TOOL:          ":/tango/16x16/mimetypes/text-html.png",
    BZR_TOOL:           ":bzr-16.png",
    APPLICATION_TOOL:   ":/tango/16x16/mimetypes/application-x-executable.png",
    # Logos
    SMALL_LOGO:         "bzr-16.png",
    MEDIUM_LOGO:        "bzr-32.png",
    LARGE_LOGO:         "bzr-48.png",
    # Actions
    ADD_ACTION:         ":/tango/16x16/actions/list-add.png",
    CANCEL_ACTION:      ":/tango/16x16/actions/process-stop.png",
    COMMIT_ACTION:      ":/tango/16x16/actions/go-last.png",
    CONFLICTS_ACTION:   ":/tango/16x16/status/software-update-urgent.png",
    DELETE_ACTION:      ":/tango/16x16/actions/edit-delete.png",
    DIFF_ACTION:        ":/tango/16x16/apps/preferences-system-windows.png",
    EDIT_ACTION:        ":/tango/16x16/apps/accessories-text-editor.png",
    LOG_ACTION:         ":/tango/16x16/actions/appointment-new.png",
    OPEN_ACTION:        ":/tango/16x16/actions/document-open.png",
    OPEN_FOLDER_ACTION: ":/tango/16x16/apps/system-file-manager.png",
    SEND_ACTION:        ":/tango/16x16/actions/mail-message-new.png",
    # Actions used on the welcome page not already defined above
    CONFIGURATION_ACTION: ":/tango/16x16/actions/contact-new.png",
    DOWNLOAD_ACTION:    ":/tango/16x16/actions/go-jump.png",
    INITIALIZE_ACTION:  ":/tango/16x16/actions/document-new.png",
    PREFERENCES_ACTION: ":/tango/16x16/categories/preferences-system.png",
    START_ACTION:       ":/tango/16x16/actions/folder-new.png",
    OPEN_LOCATION_ACTION: ":/tango/16x16/apps/internet-web-browser.png",
    # Views
    GENERIC_VIEW:       ":/tango/16x16/mimetypes/text-x-generic.png",
    }

# Cache
_icon_by_resource = {}


def icon_for_kind(kind):
    rpath = _RESOURCE_BY_KIND.get(kind, None)
    if rpath is None:
        return None
    elif rpath in _icon_by_resource:
        return _icon_by_resource[rpath]
    else:
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(rpath), QtGui.QIcon.Normal,
            QtGui.QIcon.On)
        _icon_by_resource[rpath] = icon
        return icon


def path_for_icon(kind):
    return _RESOURCE_BY_KIND.get(kind, None)


def html_status_message(status_kind, text):
    colour = {
        ERROR_STATUS: "red",
        WARNING_STATUS: "black",
        GOOD_STATUS: "black",
        }[status_kind]
    return '<img src="%s">&nbsp;&nbsp;<font color=%s>%s</font>' % (
        path_for_icon(status_kind),
        colour,
        escape(text))


def html_action_links(actions, target=None):
    """Generate action links as html.
    
    :param actions: list of action names. If an action includes a space,
      then the first text before that is assumed to be the real action
      name for the purposes of icon and tooltip lookup.
    :param target: target for actions, or None for no target
    """
    action_parts = []
    for action in actions:
        # Note: icon must have same name as action itself.
        # Perhaps we should trap a None result, mutter it and
        # ignore it? Or maybe the hint text is sufficient?
        sep = action.find(' ')
        if sep == -1:
            action_name = action
        else:
            action_name = action[:sep]
        action_icon = path_for_icon(action_name)

        # We could use gettext here to lookup localised text but it's
        # probably better to show the command itself, given it can be
        # things like 'diff -rsubmit:..-1'
        action_hint = action

        if target is None:
            action_link = action
        else:
            action_link = "%s %s" % (action, target)
        action_parts.append('<a href="%s"><img src="%s" title="%s"/></a>'
            % (action_link, action_icon, action_hint))
    return ''.join(action_parts)
