# Copyright (C) 2009 Canonical Ltd
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""Initialization of a repository and/or branch for common workspace models."""


from bzrlib.lazy_import import lazy_import
lazy_import(globals(), '''

from bzrlib import (
    builtins,
    bzrdir,
    commands,
    errors,
    osutils,
    trace,
    transport,
    )
from bzrlib.trace import note, mutter
''')


# Model identifiers
MODEL_FEATURE_BRANCHES = "feature-branches"
MODEL_SHARED_TREE = "shared-tree"
MODEL_PLAIN_BRANCH = "plain-branch"
MODEL_SHARED_REPO = "shared-repo"
MODEL_SHARED_REPO_NO_TREES = "shared-repo-no-trees"


class MissingParentDirectory(errors.BzrError):

    _fmt = "Parent directory for %(url)s does not exist."

    def __init__(self, url):
        self.url = url


class UnknownWorkspaceModel(errors.BzrError):

    _fmt = "Unknown workspace model %(model)s"

    def __init__(self, model):
        self.model = model


def init_workspace(location, model=None, format=None, create_parent_dirs=False,
    trunk_name='trunk', work_name='work', outf=None):
    """Initialise a workspace.
    
    :param location: base location of the workspace
    :param model: one of the following:
        * feature-branches - create a shared repository at location
          then create a trunk branch inside it (default)
        * shared-tree - create a (no-trees) shared repository at location,
          then create a trunk branch and separate working tree
          (lightweight checkout) inside it
        * plain-branch - create a plain branch at location; if inside a
          shared repository, that will be used, otherwise the branch will
          be a standalone branch
        * shared-repo - create a shared repository at location
        * shared-repo-no-trees - create a shared repository at location
          and configure it so branches have no trees
    :param format: storage format to use
    :param create_parent_dirs: create parent directories of location if
      they don't already exist
    :param trunk_name: name of trunk branch for feature-branches and
      shared-tree models
    :param work_name: name of trunk branch for feature-branches and
      shared-tree models
    :param outf: output stream; if None, standard output is used
    """
    # Assign defaults to parameters
    if model is None:
        model = MODEL_FEATURE_BRANCHES
    if format is None:
        format = bzrdir.format_registry.make_bzrdir('default')
    if outf is None:
        outf = sys.stdout
    mutter("initialising workspace %s using model %s" % (location, model))

    # Create the parent directories if required
    _ensure_base_of_location(location, create_parent_dirs)

    # For feature branching, we need to start with a trunk
    # branch in a shared repository
    if model == MODEL_FEATURE_BRANCHES:
        trunk_location = osutils.pathjoin(location, trunk_name)
        _init_repository(outf, location, format)
        _init_branch(outf, trunk_location, format)

    # For the shared tree model, we start with a treeless shared repository,
    # create a trunk branch and then a work checkout pointing to that trunk
    elif model == MODEL_SHARED_TREE:
        trunk_location = osutils.pathjoin(location, trunk_name)
        work_location = osutils.pathjoin(location, work_name)
        _init_repository(outf, location, format, no_trees=True)
        _init_branch(outf, trunk_location, format)
        _init_working_tree(outf, trunk_location, work_location)

    elif model == MODEL_PLAIN_BRANCH:
        _init_branch(outf, location, format)
    elif model == MODEL_SHARED_REPO:
        _init_repository(outf, location, format)
    elif model == MODEL_SHARED_REPO_NO_TREES:
        _init_repository(outf, location, format, no_trees=True)
    else:
        raise UnknownWorkspaceModel(model)


def _ensure_base_of_location(location, create_parent_dirs):
    to_transport = transport.get_transport(location)
    try:
        to_transport.ensure_base()
    except errors.NoSuchFile:
        # The path has to exist to initialize a repo or branch inside of it.
        # Raise an exception the GUI can easily trap unless we were
        # explicitly asked to create parent directories.
        if not create_parent_dirs:
            raise MissingParentDirectory(location)
        to_transport.create_prefix()


def _init_repository(outf, location, format, no_trees=False):
    """Initialise a repository."""
    old_verbosity = trace.get_verbosity_level()
    trace.set_verbosity_level(-1)
    try:
        core_cmd = _get_command_object('init-repository', outf)
        core_cmd.run(location, format=format, no_trees=no_trees)
    finally:
        trace.set_verbosity_level(old_verbosity)
    if no_trees:
        note("Created repository with treeless branches at %s" % (location,))
    else:
        note("Created repository at %s" % (location,))


def _init_branch(outf, location, format):
    """Initialise a branch."""
    old_verbosity = trace.get_verbosity_level()
    trace.set_verbosity_level(-1)
    try:
        core_cmd = _get_command_object('init', outf)
        core_cmd.run(location, format=format)
    finally:
        trace.set_verbosity_level(old_verbosity)
    note("Created branch at %s" % (location,))


def _init_working_tree(outf, branch_location, to_location):
    """Initialise a lightweight checkout to use as a working tree."""
    old_verbosity = trace.get_verbosity_level()
    trace.set_verbosity_level(-1)
    try:
        core_cmd = _get_command_object('checkout', outf)
        core_cmd.run(branch_location, to_location, lightweight=True)
    finally:
        trace.set_verbosity_level(old_verbosity)
    note("Created working tree at %s" % (to_location,))


def _get_command_object(cmd_name, outf):
    cmd_obj = commands.get_cmd_object(cmd_name)
    # We explicitly reuse the current command's output stream rather than
    # setup a fresh one here
    cmd_obj.outf = outf
    return cmd_obj
