# Copyright (C) 2009 Canonical Ltd
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import sys

from PyQt4 import QtCore, QtGui

try:
    from bzrlib.osutils import parent_directories
except ImportError:
    from bzrlib.plugins.explorer.lib.helpers import parent_directories

from bzrlib.plugins.explorer.lib.i18n import gettext, N_
from bzrlib.plugins.explorer.lib.kinds import (
    CANCEL_ACTION,
    OPEN_FOLDER_ACTION,
    EDIT_ACTION,
    icon_for_kind,
    )


class WorkingTreeBrowser(QtGui.QDockWidget):

    def __init__(self, action_callback, browse_action, *args):
        self._action_callback = action_callback
        self._browse_action = browse_action
        QtGui.QDockWidget.__init__(self, *args)
        self.setObjectName("WorkingTreeDockWidget")
        self.setWidget(self._build_ui())
        self._root = None
        self._selected_fileinfo = None
        self._current_filter = ''

    def _build_ui(self):
        # Build a filter bar
        filter_label = QtGui.QLabel(gettext("Filter:"))
        self.filter_field = QtGui.QLineEdit()
        self.filter_cancel_button = QtGui.QToolButton()
        self.filter_cancel_button.setAutoRaise(True)
        self.filter_cancel_button.setEnabled(False)
        self.filter_cancel_button.setIcon(icon_for_kind(CANCEL_ACTION))
        self.filter_cancel_button.setToolTip(gettext("Clear the filter"))
        filter_layout = QtGui.QHBoxLayout()
        filter_layout.addWidget(filter_label)
        filter_layout.addWidget(self.filter_field)
        filter_layout.addWidget(self.filter_cancel_button)

        # Init the model and view
        source_model = QtGui.QDirModel()
        # sort flags for tree view: directories first,
        # on Windows items are case-insensitive
        flags = QtCore.QDir.SortFlags(QtCore.QDir.Name|QtCore.QDir.DirsFirst)
        if sys.platform == 'win32':
            flags |= QtCore.QDir.IgnoreCase
        source_model.setSorting(flags)
        # Use a proxy model so we can filter on the path
        model = MySortFilterProxyModel(self.filter_field)
        model.setSourceModel(source_model)
        view = QtGui.QTreeView()
        view.setModel(model)

        # Hide all except the Name column
        for i in  range(1, model.columnCount()):
            view.setColumnHidden(i, True)
        if getattr(view, 'setHeaderHidden', None):
            view.setHeaderHidden(True)

        # Connect the filter to the model
        self.connect(self.filter_field, QtCore.SIGNAL("textChanged(QString)"),
            self.apply_filter)
        self.connect(self.filter_cancel_button, QtCore.SIGNAL("clicked(bool)"),
            self.do_clear_filter)

        # Build a button bar for the view
        browse_button = QtGui.QPushButton()
        browse_button.setText(self._browse_action.iconText())
        browse_button.setIcon(self._browse_action.icon())
        open_button = QtGui.QPushButton()
        open_button.setText(gettext("&Manage"))
        open_button.setIcon(icon_for_kind(OPEN_FOLDER_ACTION))
        edit_button = QtGui.QPushButton()
        edit_button.setText(gettext("&Edit"))
        edit_button.setIcon(icon_for_kind(EDIT_ACTION))
        buttons = QtGui.QDialogButtonBox()
        buttons.addButton(browse_button, QtGui.QDialogButtonBox.ActionRole)
        buttons.addButton(open_button, QtGui.QDialogButtonBox.ActionRole)
        buttons.addButton(edit_button, QtGui.QDialogButtonBox.AcceptRole)
        self.connect(browse_button, QtCore.SIGNAL("clicked(bool)"),
            self.do_browse_items)
        self.connect(open_button, QtCore.SIGNAL("clicked(bool)"),
            self.do_open_folder)
        self.connect(edit_button, QtCore.SIGNAL("clicked(bool)"),
            self.do_edit_file)
        self._edit_button = edit_button

        # Add the filter and button box to the view
        view_layout = QtGui.QVBoxLayout()
        view_layout.addLayout(filter_layout)
        view_layout.addWidget(view)
        view_layout.addWidget(buttons)
        view_panel = QtGui.QWidget()
        view_panel.setLayout(view_layout)

        # Show either the view or a message
        msg = QtGui.QTextBrowser()
        msg.setText(gettext("No working tree"))
        stacked = QtGui.QStackedWidget()
        stacked.addWidget(msg)
        stacked.addWidget(view_panel)

        # Track the selection
        self.connect(view, QtCore.SIGNAL("doubleClicked(QModelIndex)"),
            self.do_double_clicked)
        sel_model = view.selectionModel()
        self.connect(sel_model,
            QtCore.SIGNAL("currentChanged(QModelIndex,QModelIndex)"),
            self.do_current_changed)

        # Save things and return
        self.source_model = source_model
        self.model = model
        self.view = view
        self.stacked = stacked
        return stacked

    def apply_filter(self, text):
        self.model.invalidateFilter()
        if text:
            self.filter_cancel_button.setEnabled(True)
            # Filtering active. We want to see all matches and
            # this mostly works. It seems to struggle though when
            # the filter gets shorter - maybe a Qt 4.4 bug w.r.t.
            # propagating exactly what's changed to the view?
            self.view.expandAll()
        else:
            # Filtering cleared. Restore the initial view with
            # everything collapsed. (If the user has selectively
            # expanded some directories out before filtering,
            # collapsing all here implies losing that setup. We
            # could always remember the expanded directories at
            # the start of filtering and restore them at this
            # point if it proves a problem in practice.)
            self.view.collapseAll()
            self.refresh()
            self.filter_cancel_button.setEnabled(False)
        self._current_filter = text

    def do_clear_filter(self):
        self.filter_field.setText('')

    def do_browse_items(self):
        self._action_callback("list", None)

    def do_open_folder(self):
        fileinfo = self._selected_fileinfo
        if fileinfo is None:
            folder = self._root
        elif fileinfo.isDir():
            folder = fileinfo.canonicalFilePath()
        else:
            folder = fileinfo.canonicalPath()
        self._action_callback("open", folder)

    def do_edit_file(self):
        fileinfo = self._selected_fileinfo
        path = fileinfo.canonicalFilePath()
        self._action_callback("edit", unicode(path))

    def _get_fileinfo(self, model_index):
        model = model_index.model()
        if model is None:
            fileinfo = None
        else:
            source_index = model.mapToSource(model_index)
            fileinfo = self.source_model.fileInfo(source_index)
        return fileinfo

    def do_double_clicked(self, model_index):
        fileinfo = self._get_fileinfo(model_index)
        if fileinfo is not None and not fileinfo.isDir():
            path = fileinfo.canonicalFilePath()
            self._action_callback("open", unicode(path))

    def do_current_changed(self, model_index, prev_model_index):
        fileinfo = self._get_fileinfo(model_index)
        edit_is_enabled = True
        if fileinfo is None or fileinfo.isDir():
            edit_is_enabled = False
        self._edit_button.setEnabled(edit_is_enabled)
        self._selected_fileinfo = fileinfo

    def set_root(self, path):
        """Change the browser to display the tree from path down.

        :param path: if None, the view is cleared.
        """
        if path is None:
            # Show the message
            self.stacked.setCurrentIndex(0)
        else:
            # Show the tree
            self.stacked.setCurrentIndex(1)
            source_index = self.source_model.index(path)
            index = self.model.mapFromSource(source_index)
            self.view.setRootIndex(index)
            self.view.selectionModel().reset()
        self._root = path
        self.model.set_root(path)
        self._selected_fileinfo = None
        self._edit_button.setEnabled(False)
        self.do_clear_filter()

    def refresh(self):
        """Refresh the QDirModel."""
        self.source_model.refresh()


class MySortFilterProxyModel(QtGui.QSortFilterProxyModel):

    def __init__(self, filter_field, parent=None):
        QtGui.QSortFilterProxyModel.__init__(self, parent)
        self.filter_field = filter_field
        self._root = None
        self._interesting = None

    def set_root(self, root):
        self._root = root

    def invalidateFilter(self):
        self._text = unicode(self.filter_field.text())
        if self._text == '':
            self._interesting = None
        else:
            # We only want to show directories containing matching items so
            # we need to do a pass in advance working out what's of interest
            self._source = self.sourceModel()
            root_index = self._source.index(self._root)
            self._interesting = self._find_interesting(self._source,
                root_index, self._text)
        QtGui.QSortFilterProxyModel.invalidateFilter(self)

    def _find_interesting(self, source, root_index, text):
        interesting = set([''])
        directories = set()
        for source_index in self._iter_children(source, root_index):
            fileinfo = source.fileInfo(source_index)
            abspath = unicode(fileinfo.absoluteFilePath())
            relpath = abspath[len(self._root) + 1:]
            if relpath.find(text) >= 0:
                interesting.add(relpath)
                directories.update(parent_directories(relpath))
        interesting.update(directories)
        #print "\ninteresting for %s ...\n%s" % (text, "\n".join(sorted(interesting)))
        return interesting

    def _iter_children(self, source, parent):
        for i in range(0, source.rowCount(parent)):
            index = source.index(i, 0, parent)
            yield index
            for child in self._iter_children(source, index):
                yield child
        
    def filterAcceptsRow(self, source_row, source_parent):
        if self._interesting is None:
            return True
        result = False
        source_index = self._source.index(source_row, 0, source_parent)
        fileinfo = self._source.fileInfo(source_index)
        abspath = unicode(fileinfo.absoluteFilePath())
        if abspath.startswith(self._root):
            # We're inside the tree of interest
            relpath = abspath[len(self._root) + 1:]
            result = relpath in self._interesting
        elif fileinfo.isDir():
            # We're interested if it's a parent of the root
            result = self._root.startswith(abspath)
        return result
